@@ -12,7 +12,7 @@ void spatPCAcore(const mat Y, mat& Phi, mat& R,  mat& C,  mat& Lambda1, mat& Lam
     
     int p = Phi.n_rows;
     int K = Phi.n_cols;
-    int iter = 0;  
+    int iter = 0;
     arma::mat Ip, Sigtau1, temp, zero, one;
     arma::vec er(4);
     Ip.eye(p,p);
@@ -113,14 +113,14 @@ void spatPCAcore(const mat Y, mat& Phi, mat& R,  mat& C,  mat& Lambda1, mat& Lam
     }
     iter++;
     if(iter == maxit)
-      Rcpp::Rcout<<"Not converge at tau1="<<tau1<<" tau2="<<tau2<<"\n"<<std::endl;
+        Rcpp::Rcout<<"Not converge at tau1="<<tau1<<" tau2="<<tau2<<"\n"<<std::endl;
 }
 
 // [[Rcpp::depends(RcppArmadillo)]]
 // user includes
 
 using namespace arma;
-using namespace Rcpp;
+//using namespace Rcpp;
 using namespace std;
 // [[Rcpp::export]]
 arma::mat spatPCAcv_rcpp(const arma::mat Y, const int M, const int K, const arma::mat Omega, const arma::vec tau1, const arma::vec tau2, const arma::vec nk, const double rhoincre, const int maxit, const double tol){
@@ -132,23 +132,23 @@ arma::mat spatPCAcv_rcpp(const arma::mat Y, const int M, const int K, const arma
     cv.zeros(tau1.n_elem,tau2.n_elem);
     Ip.eye(p,p);
     
-
+    
     for( k = 1; k <= M; k++){
-      Ytrain = Y.rows(arma::find(nk!=(k)));
-      Yvalid = Y.rows(arma::find(nk==(k)));
-      arma::svd_econ(UPhi, SPhi, Phiold, Ytrain);
-      rho = 10*pow(SPhi[0],2.0);
-      Phi = Phiold.cols(0,K-1);
-      R = Phi;
-      C = Phi;
-      Lambda1 = 0*Phi;
-      Lambda2 = 0*Phi;
-      for(unsigned int i = 0; i < tau1.n_elem; i++){
-        for(unsigned int j = 0; j < tau2.n_elem; j++){      
-          spatPCAcore(Ytrain,Phi,R,C,Lambda1,Lambda2, Omega, tau1[i],tau2[j], rho, rhoincre, maxit,tol);       
-          cv(i,j) += pow(arma::norm(Yvalid*(Ip-Phi*Phi.t()),"fro"),2.0);
+        Ytrain = Y.rows(arma::find(nk!=(k)));
+        Yvalid = Y.rows(arma::find(nk==(k)));
+        arma::svd_econ(UPhi, SPhi, Phiold, Ytrain);
+        rho = 10*pow(SPhi[0],2.0);
+        Phi = Phiold.cols(0,K-1);
+        R = Phi;
+        C = Phi;
+        Lambda1 = 0*Phi;
+        Lambda2 = 0*Phi;
+        for(unsigned int i = 0; i < tau1.n_elem; i++){
+            for(unsigned int j = 0; j < tau2.n_elem; j++){
+                spatPCAcore(Ytrain,Phi,R,C,Lambda1,Lambda2, Omega, tau1[i],tau2[j], rho, rhoincre, maxit,tol);
+                cv(i,j) += pow(arma::norm(Yvalid*(Ip-Phi*Phi.t()),"fro"),2.0);
+            }
         }
-      }
     }
     return(cv);
 }
@@ -158,7 +158,7 @@ using namespace Rcpp;
 using namespace std;
 // [[Rcpp::export]]
 arma::mat spatPCAcv_rcpp_parallel(const arma::mat Y, const int m, const int K, const arma::mat Omega, const arma::vec tau1, const arma::vec tau2, const arma::vec nk, const double rhoincre, const int maxit, const double tol){
-      
+    
     int p = Y.n_cols;
     double rho;
     arma::mat Ytrain, Yvalid, UPhi, Phiold, Phi, R, C, Lambda1, Lambda2, Ip, eigenvalue, cv;
@@ -176,20 +176,20 @@ arma::mat spatPCAcv_rcpp_parallel(const arma::mat Y, const int m, const int K, c
     Lambda1 = 0*Phi;
     Lambda2 = 0*Phi;
     for(unsigned int i = 0; i < tau1.n_elem; i++){
-      for(unsigned int j = 0; j < tau2.n_elem; j++){      
-        spatPCAcore(Ytrain,Phi,R,C,Lambda1,Lambda2, Omega, tau1[i],tau2[j], rho, rhoincre, maxit,tol);       
-        cv(i,j) += pow(arma::norm(Yvalid*(Ip-Phi*Phi.t()),"fro"),2.0);
-      }
+        for(unsigned int j = 0; j < tau2.n_elem; j++){
+            spatPCAcore(Ytrain,Phi,R,C,Lambda1,Lambda2, Omega, tau1[i],tau2[j], rho, rhoincre, maxit,tol);
+            cv(i,j) += pow(arma::norm(Yvalid*(Ip-Phi*Phi.t()),"fro"),2.0);
+        }
     }
     
     return(cv);
 }
 
 
-// [[Rcpp::export]]
 using namespace arma;
 using namespace Rcpp;
 using namespace std;
+// [[Rcpp::export]]
 arma::mat spatPCA_rcpp(const arma::mat Y, const int K, const arma::mat Omega, const double tau1, const arma::vec l2, const double rhoincre, const int maxit, const double tol){
     
     double rho;
@@ -217,63 +217,64 @@ arma::mat spatPCA_rcpp(const arma::mat Y, const int K, const arma::mat Omega, co
 
 
 
-// [[Rcpp::export]]
+
 using namespace arma;
 using namespace Rcpp;
 using namespace std;
+// [[Rcpp::export]]
 arma::vec spatPCAcv_gamma(const arma::mat Y, const arma::mat Phi, const int M, const arma::vec gamma, const arma::vec nk ){
-  
-  int p = Y.n_cols;
-  int K = Phi.n_cols;
-  int tempL, k;
-  double totalvar, err, temp, tempSc, tempSc2;
-  arma::mat Ytrain, Yvalid, Vc, Vc2, covtrain, covvalid,covest, Ip;
-  arma::vec Sc, Sc2, Sct, Sctz, cv;
-  arma::mat eigenvalue;
-  cv.zeros(gamma.n_elem);
-  Ip.eye(p,p);
-  
-  for(k = 1; k <= M; k++){
-  Ytrain = Y.rows(arma::find(nk!=(k)));
-  Yvalid = Y.rows(arma::find(nk==(k)));
-  covtrain = arma::trans(Ytrain)*Ytrain/Ytrain.n_rows;
-  covvalid = arma::trans(Yvalid)*Yvalid  /Yvalid.n_rows;
-  totalvar = arma::trace(covtrain);
-  arma::eig_sym(Sc,Vc, trans(Phi)*covtrain*Phi);
-  tempSc2 = accu(Sc);
-  Sc2 = sort(Sc,"descend");
-  Vc2 = Vc.cols(sort_index(Sc,"descend"));
-  
-  Sct.ones(Sc2.n_elem);
-  Sctz.zeros(Sc2.n_elem);
-  
-  for(unsigned int gj = 0; gj < gamma.n_elem; gj++){
-    tempSc = tempSc2;
-    tempL = K;
-    if(Sc2[0]> gamma[gj]){
-      err = (totalvar - tempSc+K*gamma[gj])/(p-tempL);
-      temp = Sc2[tempL-1];  
-      while( temp-gamma[gj] < err){
-        if(tempL == 1){
-        err = (totalvar - Sc2[0] + gamma[gj])/(p-1);
-        break;
-      }
-      tempSc += -Sc2[tempL-1];
-      tempL--;
-      err = (totalvar - tempSc+tempL*gamma[gj])/(p-tempL);
-      temp = Sc2[tempL-1];
+    
+    int p = Y.n_cols;
+    int K = Phi.n_cols;
+    int tempL, k;
+    double totalvar, err, temp, tempSc, tempSc2;
+    arma::mat Ytrain, Yvalid, Vc, Vc2, covtrain, covvalid,covest, Ip;
+    arma::vec Sc, Sc2, Sct, Sctz, cv;
+    arma::mat eigenvalue;
+    cv.zeros(gamma.n_elem);
+    Ip.eye(p,p);
+    
+    for(k = 1; k <= M; k++){
+        Ytrain = Y.rows(arma::find(nk!=(k)));
+        Yvalid = Y.rows(arma::find(nk==(k)));
+        covtrain = arma::trans(Ytrain)*Ytrain/Ytrain.n_rows;
+        covvalid = arma::trans(Yvalid)*Yvalid  /Yvalid.n_rows;
+        totalvar = arma::trace(covtrain);
+        arma::eig_sym(Sc,Vc, trans(Phi)*covtrain*Phi);
+        tempSc2 = accu(Sc);
+        Sc2 = sort(Sc,"descend");
+        Vc2 = Vc.cols(sort_index(Sc,"descend"));
+        
+        Sct.ones(Sc2.n_elem);
+        Sctz.zeros(Sc2.n_elem);
+        
+        for(unsigned int gj = 0; gj < gamma.n_elem; gj++){
+            tempSc = tempSc2;
+            tempL = K;
+            if(Sc2[0]> gamma[gj]){
+                err = (totalvar - tempSc+K*gamma[gj])/(p-tempL);
+                temp = Sc2[tempL-1];  
+                while( temp-gamma[gj] < err){
+                    if(tempL == 1){
+                        err = (totalvar - Sc2[0] + gamma[gj])/(p-1);
+                        break;
+                    }
+                    tempSc += -Sc2[tempL-1];
+                    tempL--;
+                    err = (totalvar - tempSc+tempL*gamma[gj])/(p-tempL);
+                    temp = Sc2[tempL-1];
+                }
+                if(Sc2[0]-gamma[gj] < err)
+                    err = (totalvar)/(p);
+            }
+            else{
+                err = (totalvar)/(p);
+            }
+            
+            eigenvalue = arma::max(Sc2-(err+gamma[gj])*Sct,Sctz);
+            covest =  Phi*Vc2*diagmat(eigenvalue)*trans(Vc2)*trans(Phi);
+            cv[gj] += pow(arma::norm(covvalid-covest - err*Ip,"fro"),2.0);
+        }
     }
-    if(Sc2[0]-gamma[gj] < err)
-    err = (totalvar)/(p);
-    }
-    else{
-      err = (totalvar)/(p);
-    }
-
-    eigenvalue = arma::max(Sc2-(err+gamma[gj])*Sct,Sctz);
-    covest =  Phi*Vc2*diagmat(eigenvalue)*trans(Vc2)*trans(Phi);
-    cv[gj] += pow(arma::norm(covvalid-covest - err*Ip,"fro"),2.0);
-    }
-  }
-  return(cv);
+    return(cv);
 }
